<?php

    /**
     *  This endpoint accept three parameters
     *  i) product_id - int - Required
     *  ii) thumb_image - URL of the image
     *  iii) gallery_images - json consisting of multiple image URL's for gallery images
     * 
     *  Note: One of thumb_image or gallery_images is required. Both can be passed simultaneously.
     *  The passed images will overwrite any of existing images.
     */
    class assign_product_images extends WC_API_Resource {

        protected $base = '/assign_product_images';

        public function register_routes($routes) {
            $routes[$this->base] = [
                [
                    [$this, 'assignProductImages'],
                    WC_API_Server::READABLE | WC_API_SERVER::EDITABLE | WC_API_SERVER::ACCEPT_DATA
                ],
            ];
            return $routes;
        }

        public function assignProductImages() {
            $request = $_REQUEST;

            if (empty($request['id']) || (empty($request['thumb_image']) && empty($request['gallery_images']))) {
                echo json_encode(array("errors" => "Request data is empty."));
                exit;
            }
            $product_id = $request['id'];
            //upload thumbnail image
            if (!empty($request['thumb_image'])) {
                $thumb_image = $request['thumb_image'];
                $attachment = $this->_uploadProductImage($thumb_image);
                if ($attachment && isset($attachment['file']) && isset($attachment['type'])) {
                    $attachment_id = $this->_uploadAttachment($attachment, $product_id);
                    if ($attachment_id) {
                        set_post_thumbnail($product_id, $attachment_id);
                        $thumb_updated = 1;
                    }
                }
            }

            //update product gallery images
            if (!empty($request['gallery_images'])) {
                $gallery_images = json_decode(stripslashes($request['gallery_images']), 1);
                if (!empty($gallery_images)) {

                    $uploaded_attachments = [];

                    //loop through recieved set of image urls and upload them to wp
                    foreach ($gallery_images as $image_url) {
                        $attachment = $this->_uploadProductImage($image_url);
                        if ($attachment && isset($attachment['file']) && isset($attachment['type'])) {
                            $attachment_id = $this->_uploadAttachment($attachment, $product_id);
                            if ($attachment_id) {
                                $uploaded_attachments[] = $attachment_id;
                            }
                        }
                    }
                    if (!empty($uploaded_attachments)) {
                        $meta_value = implode(',', $uploaded_attachments);
                        update_post_meta($product_id, '_product_image_gallery', $meta_value);
                        $gallery_updated = 1;
                    }
                } elseif (empty($request['thumb_image'])) {
                    echo json_encode(array("errors" => "Request data for gallery image is invalid."));
                    exit;
                }
            }

            if (isset($gallery_updated) || isset($thumb_updated)) {
                $message = "Product's";

                if (isset($thumb_updated)) {
                    $message .=" Thumbnail ";
                }
                if (isset($gallery_updated)) {
                    if (isset($thumb_updated)) {
                        $message .="&";
                    }
                    $message .=" Gallery ";
                }

                $message .=" image(s) updated successfully";

                echo json_encode(array("success" => $message));
            } else {
                echo json_encode(array("faliure" => "There was some problem with assigning product images.please make sure image URL's are globally accessible."));
            }
            exit;
        }

        private function _uploadProductImage($image_url) {
            $file_name = basename(current(explode('?', $image_url)));
            $wp_filetype = wp_check_filetype($file_name, null);
            $parsed_url = @parse_url($image_url);

            // Check parsed URL
            if (!$parsed_url || !is_array($parsed_url)) {
                return FALSE;
            }

            // Ensure url is valid
            $image_url = str_replace(' ', '%20', $image_url);

            // Get the file
            $response = wp_safe_remote_get($image_url, array(
                'timeout' => 10
            ));

            if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
                return FALSE;
            }

            // Ensure we have a file name and type
            if (!$wp_filetype['type']) {
                $headers = wp_remote_retrieve_headers($response);
                if (isset($headers['content-disposition']) && strstr($headers['content-disposition'], 'filename=')) {
                    $disposition = end(explode('filename=', $headers['content-disposition']));
                    $disposition = sanitize_file_name($disposition);
                    $file_name = $disposition;
                } elseif (isset($headers['content-type']) && strstr($headers['content-type'], 'image/')) {
                    $file_name = 'image.' . str_replace('image/', '', $headers['content-type']);
                }
                unset($headers);
            }

            // Upload the file
            $upload = wp_upload_bits($file_name, '', wp_remote_retrieve_body($response));

            if ($upload['error']) {
                return FALSE;
            }

            // Get filesize
            $filesize = filesize($upload['file']);

            if (0 == $filesize) {
                @unlink($upload['file']);
                unset($upload);
                return FALSE;
            }

            unset($response);
            return $upload;
        }

        private function _uploadAttachment($attachment, $post_id) {
            $filetype = wp_check_filetype(basename($attachment['file']), null);
            $post_title = get_the_title($post_id);
            $postinfo = array(
                'post_mime_type' => $filetype['type'],
                'post_title' => $post_title,
                'post_content' => '',
                'post_status' => 'inherit',
            );
            $filename = $attachment['file'];
            $attach_id = wp_insert_attachment($postinfo, $filename, $post_id);
            
            update_post_meta($attach_id, "_wp_attachment_image_alt", $post_title, TRUE);
            
            return $attach_id;
        }

    }
    